"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FrameManager = void 0;
class FrameManager {
    constructor(container) {
        this.container = container;
    }
    nextFrame(timestamp) {
        try {
            const container = this.container;
            const options = container.options;
            const fpsLimit = options.fpsLimit > 0 ? options.fpsLimit : 60;
            if (container.lastFrameTime !== undefined && timestamp < container.lastFrameTime + 1000 / fpsLimit) {
                container.draw();
                return;
            }
            const deltaValue = timestamp - container.lastFrameTime;
            const delta = {
                value: deltaValue,
                factor: options.fpsLimit > 0 ? (60 * deltaValue) / 1000 : 3.6,
            };
            container.lastFrameTime = timestamp;
            container.particles.draw(delta);
            if (options.particles.move.enable && container.getAnimationStatus()) {
                container.draw();
            }
        }
        catch (e) {
            console.error("tsParticles error in animation loop", e);
        }
    }
}
exports.FrameManager = FrameManager;
